@@ -56,6 +56,8 @@ class Provider(object):
     @cvar OPSOURCE: Opsource Cloud
     @cvar NINEFOLD: Ninefold
     @cvar TERREMARK: Terremark
+    @cvar: EC2_US_WEST_OREGON: Amazon AWS US West 2 (Oregon)
+    @cvar CLOUDSTACK: CloudStack
     """
     DUMMY = 0
     EC2 = 1  # deprecated name
@@ -95,6 +97,8 @@ class Provider(object):
     SERVERLOVE = 33
     NINEFOLD = 34
     TERREMARK = 35
+    EC2_US_WEST_OREGON = 36
+    CLOUDSTACK = 37
 
 class NodeState(object):
     """
