@@ -42,6 +42,7 @@ EXPIRATION_SECONDS = 15 * 60
 
 S3_US_STANDARD_HOST = 's3.amazonaws.com'
 S3_US_WEST_HOST = 's3-us-west-1.amazonaws.com'
+S3_US_WEST_OREGON_HOST = 's3-us-west-2.amazonaws.com'
 S3_EU_WEST_HOST = 's3-eu-west-1.amazonaws.com'
 S3_AP_SOUTHEAST_HOST = 's3-ap-southeast-1.amazonaws.com'
 S3_AP_NORTHEAST_HOST = 's3-ap-northeast-1.amazonaws.com'
@@ -478,6 +479,14 @@ class S3USWestStorageDriver(S3StorageDriver):
     connectionCls = S3USWestConnection
     ex_location_name = 'us-west-1'
 
+class S3USWestOregonConnection(S3Connection):
+    host = S3_US_WEST_OREGON_HOST
+
+class S3USWestOregonStorageDriver(S3StorageDriver):
+    name = 'Amazon S3 (us-west-2)'
+    connectionCls = S3USWestOregonConnection
+    ex_location_name = 'us-west-2'
+
 class S3EUWestConnection(S3Connection):
     host = S3_EU_WEST_HOST
 
