@@ -106,6 +106,10 @@ class OpenStack_1_0_Tests(unittest.TestCase, TestCaseMixin):
         else:
             self.fail('test should have thrown')
 
+    def test_list_locations(self):
+        locations = self.driver.list_locations()
+        self.assertEqual(len(locations), 1)
+
     def test_list_nodes(self):
         OpenStackMockHttp.type = 'EMPTY'
         ret = self.driver.list_nodes()
@@ -676,6 +680,10 @@ class OpenStack_1_1_MockHttp(MockHttpTestCase):
     auth_fixtures = OpenStackFixtures()
     json_content_headers = {'content-type': 'application/json; charset=UTF-8'}
 
+    def _v2_0_tokens(self, method, url, body, headers):
+        body = self.auth_fixtures.load('_v2_0__auth.json')
+        return (httplib.OK, body, self.json_content_headers, httplib.responses[httplib.OK])
+
     def _v1_0_(self, method, url, body, headers):
         headers = {
             'x-auth-token': 'FE011C19-CF86-4F87-BE5D-9229145D7A06',
@@ -683,41 +691,41 @@ class OpenStack_1_1_MockHttp(MockHttpTestCase):
         }
         return (httplib.NO_CONTENT, "", headers, httplib.responses[httplib.NO_CONTENT])
 
-    def _servers_detail(self, method, url, body, headers):
+    def _v1_1_slug_servers_detail(self, method, url, body, headers):
         body = self.fixtures.load('_servers_detail.json')
         return (httplib.OK, body, self.json_content_headers, httplib.responses[httplib.OK])
 
-    def _flavors_detail(self, method, url, body, headers):
+    def _v1_1_slug_flavors_detail(self, method, url, body, headers):
         body = self.fixtures.load('_flavors_detail.json')
         return (httplib.OK, body, self.json_content_headers, httplib.responses[httplib.OK])
 
-    def _images_detail(self, method, url, body, headers):
+    def _v1_1_slug_images_detail(self, method, url, body, headers):
         body = self.fixtures.load('_images_detail.json')
         return (httplib.OK, body, self.json_content_headers, httplib.responses[httplib.OK])
 
-    def _servers(self, method, url, body, headers):
+    def _v1_1_slug_servers(self, method, url, body, headers):
         body = self.fixtures.load('_servers.json')
         return (httplib.OK, body, self.json_content_headers, httplib.responses[httplib.OK])
 
-    def _servers_12065_action(self, method, url, body, headers):
+    def _v1_1_slug_servers_12065_action(self, method, url, body, headers):
         if method != "POST":
             self.fail('HTTP method other than POST to action URL')
 
         return (httplib.ACCEPTED, "", {}, httplib.responses[httplib.ACCEPTED])
 
-    def _servers_12064_action(self, method, url, body, headers):
+    def _v1_1_slug_servers_12064_action(self, method, url, body, headers):
         if method != "POST":
             self.fail('HTTP method other than POST to action URL')
 
         return (httplib.ACCEPTED, "", {}, httplib.responses[httplib.ACCEPTED])
 
-    def _servers_12065(self, method, url, body, headers):
+    def _v1_1_slug_servers_12065(self, method, url, body, headers):
         if method == "DELETE":
             return (httplib.ACCEPTED, "", {}, httplib.responses[httplib.ACCEPTED])
         else:
             raise NotImplementedError()
 
-    def _servers_12064(self, method, url, body, headers):
+    def _v1_1_slug_servers_12064(self, method, url, body, headers):
         if method == "GET":
             body = self.fixtures.load('_servers_12064.json')
             return (httplib.OK, body, self.json_content_headers, httplib.responses[httplib.OK])
@@ -729,12 +737,12 @@ class OpenStack_1_1_MockHttp(MockHttpTestCase):
         else:
             raise NotImplementedError()
 
-    def _servers_12062(self, method, url, body, headers):
+    def _v1_1_slug_servers_12062(self, method, url, body, headers):
         if method == "GET":
             body = self.fixtures.load('_servers_12064.json')
             return (httplib.OK, body, self.json_content_headers, httplib.responses[httplib.OK])
 
-    def _servers_12063_metadata(self, method, url, body, headers):
+    def _v1_1_slug_servers_12063_metadata(self, method, url, body, headers):
         if method == "GET":
             body = self.fixtures.load('_servers_12063_metadata_two_keys.json')
             return (httplib.OK, body, self.json_content_headers, httplib.responses[httplib.OK])
@@ -742,26 +750,36 @@ class OpenStack_1_1_MockHttp(MockHttpTestCase):
             body = self.fixtures.load('_servers_12063_metadata_two_keys.json')
             return (httplib.OK, body, self.json_content_headers, httplib.responses[httplib.OK])
 
-    def _flavors_7(self, method, url, body, headers):
+    def _v1_1_slug_flavors_7(self, method, url, body, headers):
         if method == "GET":
             body = self.fixtures.load('_flavors_7.json')
             return (httplib.OK, body, self.json_content_headers, httplib.responses[httplib.OK])
         else:
             raise NotImplementedError()
 
-    def _images_13(self, method, url, body, headers):
+    def _v1_1_slug_images_13(self, method, url, body, headers):
         if method == "GET":
             body = self.fixtures.load('_images_13.json')
             return (httplib.OK, body, self.json_content_headers, httplib.responses[httplib.OK])
         else:
             raise NotImplementedError()
 
-    def _images_DELETEUUID(self, method, url, body, headers):
+    def _v1_1_slug_images_DELETEUUID(self, method, url, body, headers):
         if method == "DELETE":
             return (httplib.ACCEPTED, "", {}, httplib.responses[httplib.ACCEPTED])
         else:
             raise NotImplementedError()
 
+class OpenStack_1_1_Auth_2_0_Tests(OpenStack_1_1_Tests):
+    driver_kwargs = {'ex_force_auth_version': '2.0'}
+
+    def setUp(self):
+        self.driver_klass.connectionCls.conn_classes = (OpenStack_1_1_MockHttp, OpenStack_1_1_MockHttp)
+        self.driver_klass.connectionCls.auth_url = "https://auth.api.example.com/v2.0/"
+        OpenStack_1_1_MockHttp.type = None
+        self.driver = self.create_driver()
+        clear_pricing_data()
+        self.node = self.driver.list_nodes()[1]
 
 if __name__ == '__main__':
     sys.exit(unittest.main())
